@@ -9,27 +9,10 @@
 #include "fuse.h"
 #include "fuse_lowlevel.h"
 
-struct fuse_chan;
-struct fuse_ll;
-
-struct fuse_session {
-	struct fuse_session_ops op;
-
-	int (*receive_buf)(struct fuse_session *se, struct fuse_buf *buf,
-			   struct fuse_chan **chp);
-
-	void (*process_buf)(void *data, const struct fuse_buf *buf,
-			    struct fuse_chan *ch);
-
-	void *data;
-
-	volatile int exited;
-
-	struct fuse_chan *ch;
-};
+struct mount_opts;
 
 struct fuse_req {
-	struct fuse_ll *f;
+	struct fuse_session *se;
 	uint64_t unique;
 	int ctr;
 	pthread_mutex_t lock;
@@ -58,19 +41,13 @@ struct fuse_notify_req {
 	struct fuse_notify_req *prev;
 };
 
-struct fuse_ll {
+struct fuse_session {
+	char *mountpoint;
+	volatile int exited;
+	int fd;
+	struct mount_opts *mo;
 	int debug;
-	int allow_root;
-	int atomic_o_trunc;
-	int no_remote_posix_lock;
-	int no_remote_flock;
-	int big_writes;
-	int splice_write;
-	int splice_move;
-	int splice_read;
-	int no_splice_write;
-	int no_splice_move;
-	int no_splice_read;
+	int deny_others;
 	struct fuse_lowlevel_ops op;
 	int got_init;
 	struct cuse_data *cuse_data;
@@ -85,46 +62,69 @@ struct fuse_ll {
 	int broken_splice_nonblock;
 	uint64_t notify_ctr;
 	struct fuse_notify_req notify_list;
+	size_t bufsize;
+	int error;
 };
 
-struct fuse_cmd {
-	char *buf;
-	size_t buflen;
-	struct fuse_chan *ch;
+struct fuse_chan {
+	pthread_mutex_t lock;
+	int ctr;
+	int fd;
 };
 
-struct fuse *fuse_new_common(struct fuse_chan *ch, struct fuse_args *args,
-			     const struct fuse_operations *op,
-			     size_t op_size, void *user_data, int compat);
+/**
+ * Filesystem module
+ *
+ * Filesystem modules are registered with the FUSE_REGISTER_MODULE()
+ * macro.
+ *
+ */
+struct fuse_module {
+	char *name;
+	fuse_module_factory_t factory;
+	struct fuse_module *next;
+	struct fusemod_so *so;
+	int ctr;
+};
 
-int fuse_sync_compat_args(struct fuse_args *args);
+/* ----------------------------------------------------------- *
+ * Channel interface (when using -o clone_fd)		       *
+ * ----------------------------------------------------------- */
 
-struct fuse_chan *fuse_kern_chan_new(int fd);
+/**
+ * Obtain counted reference to the channel
+ *
+ * @param ch the channel
+ * @return the channel
+ */
+struct fuse_chan *fuse_chan_get(struct fuse_chan *ch);
 
-struct fuse_session *fuse_lowlevel_new_common(struct fuse_args *args,
-					const struct fuse_lowlevel_ops *op,
-					size_t op_size, void *userdata);
-
-void fuse_kern_unmount_compat22(const char *mountpoint);
-int fuse_chan_clearfd(struct fuse_chan *ch);
+/**
+ * Drop counted reference to a channel
+ *
+ * @param ch the channel
+ */
+void fuse_chan_put(struct fuse_chan *ch);
 
+struct mount_opts *parse_mount_opts(struct fuse_args *args);
+void destroy_mount_opts(struct mount_opts *mo);
+void fuse_mount_version(void);
+unsigned get_max_read(struct mount_opts *o);
 void fuse_kern_unmount(const char *mountpoint, int fd);
-int fuse_kern_mount(const char *mountpoint, struct fuse_args *args);
+int fuse_kern_mount(const char *mountpoint, struct mount_opts *mo);
 
 int fuse_send_reply_iov_nofree(fuse_req_t req, int error, struct iovec *iov,
 			       int count);
 void fuse_free_req(fuse_req_t req);
 
-
-struct fuse *fuse_setup_common(int argc, char *argv[],
-			       const struct fuse_operations *op,
-			       size_t op_size,
-			       char **mountpoint,
-			       int *multithreaded,
-			       int *fd,
-			       void *user_data,
-			       int compat);
-
 void cuse_lowlevel_init(fuse_req_t req, fuse_ino_t nodeide, const void *inarg);
 
 int fuse_start_thread(pthread_t *thread_id, void *(*func)(void *), void *arg);
+
+int fuse_session_receive_buf_int(struct fuse_session *se, struct fuse_buf *buf,
+				 struct fuse_chan *ch);
+void fuse_session_process_buf_int(struct fuse_session *se,
+				  const struct fuse_buf *buf, struct fuse_chan *ch);
+
+struct fuse *fuse_new_31(struct fuse_args *args, const struct fuse_operations *op,
+		      size_t op_size, void *private_data);
